from sanic.response import HTTPResponse
from json import dumps
from jsonclasses import JSONEncoder


def json(
    body,
    status=200,
    headers=None,
    content_type='application/json'
):
    return HTTPResponse(
        dumps(body, cls=JSONEncoder),
        headers=headers,
        status=status,
        content_type=content_type
    )


def data(
    body,
    status=200,
    headers=None,
    content_type='application/json'
):
    return json(
        {'data': body},
        status=status,
        headers=headers,
        content_type=content_type
    )


def empty(
    body,
    status=200,
    headers=None,
    content_type='application/json'
):
    return json(
        {},
        status=status,
        headers=headers,
        content_type=content_type
    )



from functools import wraps
from .middlewares import (only_accept_middleware, only_respond_middleware,
                          only_handle_middleware, only_accept_json_middleware,
                          only_respond_json_middleware)


def only_accept(content_types):
    def only_accept_content_type(handler):
        @wraps(handler)
        async def wrapped(request, *args, **kwargs):
            await only_accept_middleware(content_types)(request)
            return await handler(request, *args, **kwargs)
        return wrapped
    return only_accept_content_type


def only_respond(content_types):
    def only_respond_content_type(handler):
        @wraps(handler)
        async def wrapped(request, *args, **kwargs):
            await only_respond_middleware(content_types)(request)
            return await handler(request, *args, **kwargs)
        return wrapped
    return only_respond_content_type


def only_handle(content_types):
    def only_handle_content_type(handler):
        @wraps(handler)
        async def wrapped(request, *args, **kwargs):
            await only_handle_middleware(content_types)(request)
            return await handler(request, *args, **kwargs)
        return wrapped
    return only_handle_content_type


def only_accept_json(handler):
    @wraps(handler)
    async def wrapped(request, *args, **kwargs):
        await only_accept_json_middleware(request)
        return await handler(request, *args, **kwargs)
    return wrapped


def only_respond_json(handler):
    @wraps(handler)
    async def wrapped(request, *args, **kwargs):
        await only_respond_json_middleware(request)
        return await handler(request, *args, **kwargs)
    return wrapped


def only_handle_json(handler):
    @wraps(handler)
    async def wrapped(request, *args, **kwargs):
        await only_accept_json_middleware(request)
        await only_respond_json_middleware(request)
        return await handler(request, *args, **kwargs)
    return wrapped



import pathlib
from setuptools import setup, find_packages

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

setup(name='jsonclasses-sanic',
      version='1.0.0',
      description='JSON Classes integration with sanic.',
      long_description=README,
      long_description_content_type="text/markdown",
      author='Wiosoft Crafts',
      author_email='wiosoftvictor@163.com',
      license='MIT',
      packages=find_packages(exclude=("tests")),
      zip_safe=False,
      url='https://github.com/Wiosoft-Crafts/jsonclasses-sanic',
      include_package_data=True,
      python_requires='>=3.8',
      install_requires=[])



from sanic.request import Request
from .exceptions import UnsupportedMediaTypeException, NotAcceptableException


def only_accept_middleware(content_types):
    if type(content_types) is str:
        content_types = [content_types]

    async def only_accept_content_types(request: Request):
        if request.method in ['POST', 'PATCH', 'PUT']:
            content_type = request.headers['content-type']
            for supported_content_type in content_types:
                if content_type.startswith(supported_content_type):
                    return
            raise UnsupportedMediaTypeException(content_type)
    return only_accept_content_types


only_accept_json_middleware = only_accept_middleware('application/json')


def only_respond_middleware(content_types):
    if type(content_types) is str:
        content_types = [content_types]

    async def only_respond_content_types(request: Request):
        accept = request.headers['accept']
        if accept is None:
            return
        if '*/*' in accept:
            return
        for content_type in content_types:
            if content_type in accept:
                return
        raise NotAcceptableException(accept)
    return only_respond_content_types


only_respond_json_middleware = only_respond_middleware('application/json')


def only_handle_middleware(content_types):
    async def only_handle_content_types(request: Request):
        await only_accept_middleware(content_types)(request)
        await only_respond_middleware(content_types)(request)
    return only_handle_content_types


only_handle_json_middleware = only_handle_middleware('application/json')



from traceback import extract_tb
from os import getcwd, path
from sanic.request import Request
from sanic.response import json
from sanic.exceptions import SanicException
from jsonclasses.exceptions import ObjectNotFoundException, ValidationException


class UnsupportedMediaTypeException(Exception):

    def __init__(self, content_type: str):
        self.message = f'Content-Type \'{content_type}\' is not supported.'
        super().__init__(self.message)


class NotAcceptableException(Exception):

    def __init__(self, accept: str):
        self.message = f'Accept \'{accept}\' is not supported.'
        super().__init__(self.message)


def remove_none(obj: dict) -> dict:
    return {k: v for k, v in obj.items() if v is not None}


def exception_handler(request: Request, exception: Exception):
    code = exception.status_code if isinstance(
        exception, SanicException) else 500
    code = 415 if isinstance(
        exception, UnsupportedMediaTypeException) else code
    code = 406 if isinstance(exception, NotAcceptableException) else code
    code = 404 if isinstance(exception, ObjectNotFoundException) else code
    code = 400 if isinstance(exception, ValidationException) else code
    if request.app.debug:
        if code == 500:
            print(exception)
            return json({
                'error': remove_none({
                    'type': 'Internal Server Error',
                    'message': 'There is an internal server error.',
                    'error_type': exception.__class__.__name__,
                    'error_message': str(exception),
                    'fields': (exception.keypath_messages
                            if isinstance(exception, ValidationException)
                            else None),
                    'traceback': [f'file {path.relpath(f.filename, getcwd())}:{f.lineno} in {f.name}' for f in extract_tb(exception.__traceback__)],  # noqa: E501
                })
            }, status=code)
        else:
            return json({
                'error': remove_none({
                    'type': exception.__class__.__name__,
                    'message': str(exception),
                    'fields': (exception.keypath_messages
                            if isinstance(exception, ValidationException)
                            else None),
                    'traceback': [f'file {path.relpath(f.filename, getcwd())}:{f.lineno} in {f.name}' for f in extract_tb(exception.__traceback__)],  # noqa: E501
                })
            }, status=code)
    else:
        if code == 500:
            print(exception)
            return json({
                'error': remove_none({
                    'type': 'Internal Server Error',
                    'message': 'There is an internal server error.'
                })
            }, status=500)
        else:
            return json({
                'type': exception.__class__.__name__,
                'message': str(exception),
                'fields': (exception.keypath_messages
                        if isinstance(exception, ValidationException)
                        else None)
            }, status=code)
